library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity stopwatch_tb is
end entity;

architecture sim of stopwatch_tb is
	component stopwatch is
		port(clk, reset : in std_logic; -- clk is "fast enough". reset active high.
		     hundradelspuls : in std_logic;
		     muxfrekvens : in std_logic; -- tic for multiplexing the display.
		     start_stopp : in std_logic;
		     nollstallning : in std_logic; -- reset counters. Active high
		     visningslage : in std_logic; -- 1:show min/sec. 0: show sec/centisec
		     display : out unsigned(1 downto 0); -- 0=right most
		     digit : out std_logic_vector(0 to 7); -- 0=top, 6=middle, 7=DP
		     raknar : out std_logic);
	end component;
	-- DUT I/O:
	signal clk,reset : std_logic := '0';
	signal hundradelspuls : std_logic := '0';
	signal muxfrekvens : std_logic := '0'; -- tic for multiplexing the display.
	signal start_stopp : std_logic;
	signal nollstallning : std_logic; -- reset counters. Active high
	signal visningslage : std_logic; -- 1:show min/sec. 0: show sec/centisec
	signal display : unsigned(1 downto 0);
	signal digit : std_logic_vector(0 to 7); -- 0=top, 6=middle, 7=DP
	signal raknar : std_logic;
	
	signal done : boolean := false;
	signal bcd : unsigned(3 downto 0); -- digit converted to unsigned.
	signal BCD3, BCD2, BCD1, BCD0, BCD30 : integer;
	signal dp_pos : integer;
begin
	-- scale up time a factor 1000. Except the clock.
	clk <= not clk after 100 ns when not done; -- 5 MHz rather than 8 MHz
	reset <= '1', '0' after 2 us;
	hundradelspuls <= not hundradelspuls after 5 us when not done; -- 100 kHz rather than 100 Hz
	muxfrekvens <= not muxfrekvens after 300 ns when not done; -- 1.67 MHz. All screen updated within 2.4 us.
	
	process begin
		visningslage <= '0'; -- show ss:cs (cs = centiseconds = second)
		nollstallning <= '1', '0' after 50 us; -- reset
		start_stopp <= '0';
		report "Init with reset" severity note;
		wait for 100 us;
		assert BCD30 = 0000 report "1. BCD not 0 after reset               :-(" severity error;
		assert raknar = '0' report "2. raknar is not 0 as expected         :-(" severity error;
		
		-- Run for 1.5 s
		start_stopp <= '1', '0' after 50 us;
		report "Starting" severity note;
		wait for 1500 us;
		assert BCD30 = 0150 report "3. BCD not 150 after 1.5 s             :-(" severity error;
		assert raknar = '1' report "4. raknar is not 1 as expected         :-(" severity error;
		
		-- Reset again.
		nollstallning <= '1', '0' after 50 us;
		report "zero" severity note;
		wait for 73 ms; -- 73 sec - 50 ms = 72.95 s => should show 12:95
		assert BCD30 = 1295 report "5. BCD not 12.95, 72.95 s after reset  :-(" severity error;
		assert raknar = '1' report "6. raknar is not 0 as expected         :-(" severity error;
		
		start_stopp <= '1', '0' after 50 us, '1' after 1500 us, '0' after 2300 us;
		report "Stop/start" severity note;
		wait for 10 ms; --72.95 s + (10 s - 1.5 s) = 81.45 s
		assert BCD30 = 2145 report "7. BCD not 21.45                       :-(" severity error;
		
		visningslage <= '1'; -- show mm:ss
		report "Switch to mm:ss view" severity note;
		wait for 10 us; -- wait for screen to update. 81.46 s = 01:21.46
		assert BCD30 = 0121 report "8. BCD not 01:21                       :-(" severity error;
		
		report "Now simulate one hour in x1000 speed. Done at 3.69 sec" severity note;
		
		wait for 1800 ms; -- half an hour.
		assert BCD30 = 3121 report "9. BCD not 31:21                       :-(" severity error;
		
		wait for 1800 ms; -- half an hour.
		assert BCD30 = 0121 report "10. BCD not 01:21 (+ 1h)               :-(" severity error;
		
		start_stopp <= '1', '0' after 50 us;
		report "After one hour: Stop." severity note;
		wait for 10 ms;
		
		done <= true;
		wait;
	end process;
	
	-- Design under test:
	DUT : stopwatch port map (
	  clk => clk,
	  reset => reset,
		hundradelspuls => hundradelspuls,
		muxfrekvens => muxfrekvens,
		start_stopp => start_stopp,
		nollstallning => nollstallning,
		visningslage => visningslage,
		display => display,
		digit => digit,
		raknar => raknar);
	
	-- Drivers (convert display+digit to an integer in range 0-9999):
	bcd <= "0000" when digit(0 to 6) = "1111110" else
	       "0001" when digit(0 to 6) = "0110000" else
	       "0010" when digit(0 to 6) = "1101101" else
	       "0011" when digit(0 to 6) = "1111001" else
	       "0100" when digit(0 to 6) = "0110011" else
	       "0101" when digit(0 to 6) = "1011011" else
	       "0110" when digit(0 to 6) = "1011111" else
	       "0111" when digit(0 to 6) = "1110000" else
	       "1000" when digit(0 to 6) = "1111111" else
	       "1001" when digit(0 to 6) = "1111011" else
	       "0000" when digit(0 to 6) = "0000000" else
	       "XXXX";
	process(clk) begin
		if display = 0 then BCD0 <= to_integer(bcd); end if; -- This means e.g. "Let it be 5 now, and -1 after 5 us". But before 5 us, it is expected to be updated again
		if display = 1 then BCD1 <= to_integer(bcd); end if;
		if display = 2 then BCD2 <= to_integer(bcd); end if;
		if display = 3 then BCD3 <= to_integer(bcd); end if;
		if digit(7) = '1' then dp_pos <= to_integer(display); end if;
	end process;
	BCD30 <= 1000*BCD3 + 100*BCD2 + 10*BCD1 + 1*BCD0;
end architecture;

